/* SPDX-License-Identifier: GPL-2.0 */
/*
 *  Kylin security modules
 *
 *  This file contains the Kylin security hook defination
 *
 *  Authors:  penghuaihe <penghuaihe@kylinos.cn>
 *
 *  Copyright (C) 2024, KylinSoft Corporation. All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2,
 *  as published by the Free Software Foundation.
 */

#ifndef _KSAF_HOOKS_H_
#define _KSAF_HOOKS_H_

#include <linux/errno.h>
#include <linux/list.h>
#include <linux/spinlock.h>
#include <linux/init.h>
#include <linux/security.h>
#include <linux/lsm_hooks.h>

struct net_device;
struct drm_connector;
struct device;
struct path;
struct pci_dev;
struct block_device;
struct usb_device;
struct cdrom_device_info;
struct usb_driver;
struct usb_interface;

union ksaf_lsm_list_options {
	#define LSM_HOOK(RET, DEFAULT, NAME, ...) RET (*NAME)(__VA_ARGS__);
	#include "lsm_hook_defs.h"
	#include "ksaf_ext_hook_defs.h"
	#undef LSM_HOOK
};

struct ksaf_lsm_hook_heads {
	#define LSM_HOOK(RET, DEFAULT, NAME, ...) struct hlist_head NAME;
	#include "lsm_hook_defs.h"
	#include "ksaf_ext_hook_defs.h"
	#undef LSM_HOOK
} __randomize_layout;

struct ksaf_lsm_hook_list {
	struct hlist_node		list;
	struct hlist_head		*head;
	union ksaf_lsm_list_options hook;
	const char			*lsm;
} __randomize_layout;

extern struct ksaf_lsm_hook_heads ksaf_lsm_hook_heads;
#define KSAF_LSM_HOOK_INIT(HEAD, HOOK) \
	{ .head = &ksaf_lsm_hook_heads.HEAD, .hook = { .HEAD = HOOK } }

#ifdef CONFIG_SECURITY_KSAF
int ksaf_module_lsm_hook_register(struct ksaf_lsm_hook_list *hooks, int count);
int ksaf_module_lsm_hook_unregister(struct ksaf_lsm_hook_list *hooks, int count);

int ksaf_module_delete(struct module *mod);
int ksaf_getprocattr(struct task_struct *p, const char *name, char **value);
int ksaf_rewrite_cred(struct task_struct *tsk, const char *name,
		      void *value, size_t size);

void ksaf_inode_post_rename(struct inode *old_dir, struct dentry *old_dentry,
			    struct inode *new_dir, struct dentry *new_dentry,
			    unsigned int flags);
void ksaf_inode_copy_security(struct inode *dest, struct inode *src);
int ksaf_inode_hide(u64 ino, struct inode *parent);

int ksaf_cap_capable(const struct cred *cred,
		     struct user_namespace *targ_ns,
		     int cap, unsigned int opts);
void ksaf_kfile_close(struct file *filp);
int ksaf_dentry_mark_dirty(struct dentry *dentry, const char *op_type);
void ksaf_inode_notify(struct inode *dir, struct dentry *dentry);
void ksaf_kmemfd_create(struct inode *inode);
int ksaf_devctl_add_usbdevinfo(struct device *dev, struct usb_driver *driver,
			       struct usb_interface *intf);
void ksaf_devctl_notify_user(int plug, unsigned int pid, unsigned int vid,
			     int perm, char *serial, char *manufacture,
			     char *product);
void ksaf_cred_set_hash(struct linux_binprm *bprm);
int ksaf_skip_inode_setxattr(struct mnt_idmap *idmap, struct dentry *dentry,
			     const char *name, const void *value, size_t size,
			     int flags);
int ksaf_file_pre_open(struct file *file);
#else /* CONFIG_SECURITY_KSAF */
static inline int
ksaf_module_lsm_hook_register(struct ksaf_lsm_hook_list *hooks, int count)
{
	return 0;
}

static inline int
ksaf_module_lsm_hook_unregister(struct ksaf_lsm_hook_list *hooks, int count)
{
	return 0;
};

static inline int ksaf_module_delete(struct module *mod)
{
	return 0;
}

static inline int ksaf_getprocattr(struct task_struct *p, const char *name, char **value)
{
	return 0;
}

static inline int
ksaf_rewrite_cred(struct task_struct *tsk, const char *name, void *value, size_t size)
{
	return 0;
}

static inline void
ksaf_inode_post_rename(struct inode *old_dir, struct dentry *old_dentry,
		       struct inode *new_dir, struct dentry *new_dentry,
		       unsigned int flags)
{}

static inline void ksaf_inode_copy_security(struct inode *dest, struct inode *src)
{}

static inline int ksaf_inode_hide(u64 ino, struct inode *parent)
{
	return 0;
}

static inline int ksaf_cap_capable(const struct cred *cred,
				   struct user_namespace *targ_ns,
				   int cap, unsigned int opts)
{
	return -EPERM;
}

static inline void ksaf_kfile_close(struct file *filp)
{}

static inline int ksaf_dentry_mark_dirty(struct dentry *dentry, const char *op_type)
{
	return 0;
}

static inline void ksaf_inode_notify(struct inode *dir, struct dentry *dentry)
{}

static inline void ksaf_kmemfd_create(struct inode *inode)
{}

static inline int
ksaf_devctl_add_usbdevinfo(struct device *dev, struct usb_driver *driver,
			   struct usb_interface *intf)
{
	return 0;
}

static inline void
ksaf_devctl_notify_user(int plug, unsigned int pid, unsigned int vid, int perm,
			char *serial, char *manufacture, char *product)
{}

static inline void ksaf_cred_set_hash(struct linux_binprm *bprm)
{}

static inline int
ksaf_skip_inode_setxattr(struct mnt_idmap *idmap, struct dentry *dentry,
			 const char *name, const void *value, size_t size,
			 int flags)
{
	return 0;
}

static inline int ksaf_file_pre_open(struct file *file)
{
	return 0;
}

#endif /* CONFIG_SECURITY_KSAF */
#endif
